@@ -17,10 +17,10 @@ use std::error::Error as StdError;
 use std::{fmt, fs};
 use std::io::{self, Read};
 use std::ops::{Deref, DerefMut};
-use std::path::{Path, PathBuf};
+use std::path::Path;
 
 use formdata::UploadedFile;
-use iron::{headers, Headers, mime, Request};
+use iron::{headers, mime, Request};
 use iron::mime::Mime;
 use iron::typemap::Key;
 use plugin::{Pluggable, Plugin};
@@ -144,49 +144,39 @@ impl fmt::Debug for Value {
 ///
 /// Files are streamed to disk because they may not fit in memory.
 #[derive(Clone, Debug, PartialEq)]
-pub struct File {
-    path: PathBuf,
-    filename: Option<String>,
-    content_type: Mime,
-    size: usize,
-}
+pub struct File(UploadedFile);
 
 impl File {
     /// Attempts to open the file in read-only mode.
     pub fn open(&self) -> io::Result<fs::File> {
-        fs::File::open(&self.path)
+        fs::File::open(&self.0.path)
     }
 
     /// The path to the temporary file where the data was saved.
     pub fn path(&self) -> &Path {
-        self.path.as_path()
+        self.0.path.as_path()
     }
 
     /// The filename that was specified in the request, unfiltered. It may or may not be legal on
     /// the local filesystem.
     pub fn filename(&self) -> Option<&str> {
-        self.filename.as_ref().map(|f| &**f)
+        self.0.filename.as_ref().map(|f| &**f)
     }
 
     /// The unvalidated `Content-Type` that was specified in the request data.
     pub fn content_type(&self) -> &Mime {
-        &self.content_type
+        &self.0.content_type
     }
 
     /// The size of the file, in bytes.
     pub fn size(&self) -> usize {
-        self.size
+        self.0.size
     }
 }
 
 impl From<UploadedFile> for File {
     fn from(file: UploadedFile) -> File {
-        File {
-            path: file.path,
-            filename: file.filename,
-            content_type: file.content_type,
-            size: file.size,
-        }
+        File(file)
     }
 }
 
@@ -520,11 +510,13 @@ impl ToParams for Json {
 }
 
 fn try_parse_multipart(req: &mut Request, map: &mut Map) -> Result<(), ParamsError> {
-    use formdata::Error::*;
+    let boundary = match formdata::get_multipart_boundary(&req.headers) {
+        Ok(boundary) => boundary,
+        Err(_) => return Ok(()),
+    };
 
-    let form_data = match formdata::parse_multipart(&mut IronRequest(req)) {
+    let form_data = match formdata::parse_multipart(&mut req.body, boundary) {
         Ok(form_data) => form_data,
-        Err(NoRequestContentType) | Err(NotMultipart) | Err(NotFormData) => return Ok(()),
         Err(err) => return Err(FormDataError(err)),
     };
 
@@ -558,15 +550,3 @@ fn try_parse_url_encoded<'a, 'b, P>(req: &mut Request<'a, 'b>, map: &mut Map)
 
     Ok(())
 }
-
-struct IronRequest<'c, 'a: 'c, 'b: 'a>(&'c mut Request<'a, 'b>);
-
-impl<'c, 'a: 'c, 'b: 'a> formdata::Request for IronRequest<'c, 'a, 'b> {
-    fn headers(&self) -> &Headers {
-        &self.0.headers
-    }
-
-    fn read_mut(&mut self) -> &mut Read {
-        &mut self.0.body
-    }
-}
